// -*- C++ -*-
/*!
 * @file DataInPort.h
 * @brief RTC::Port implementation for Data InPort
 * @date $Date: 2007/01/14 22:57:48 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: DataInPort.h,v 1.4 2007/01/14 22:57:48 n-ando Exp $
 *
 */

/*
 * $Log: DataInPort.h,v $
 * Revision 1.4  2007/01/14 22:57:48  n-ando
 * A bug fix about template argument for buffer-type in constructor.
 *
 * Revision 1.3  2007/01/12 14:29:48  n-ando
 * A trivial bug fix.
 *
 * Revision 1.2  2007/01/06 17:43:39  n-ando
 * The behavior on notify_connect() and notify_disconnect() are now
 * implemented in protected functions(ex. publisherInterfaces()).
 *
 * Revision 1.1  2006/12/02 18:27:49  n-ando
 * The first commitment.
 *
 *
 */

#ifndef DataInPort_h
#define DataInPort_h

#include <rtm/PortBase.h>
#include <rtm/BufferBase.h>
#include <rtm/InPortCorbaProvider.h>
#include <rtm/OutPortCorbaConsumer.h>
#include <rtm/NVUtil.h>
#include <rtm/InPort.h>
namespace RTC
{
  /*!
   * @if jp
   * @class DataInPort
   * @brief InPort  Port
   * @else
   * @class DataInPort
   * @brief InPort abstruct class
   * @endif
   */

  class DataInPort
    : public virtual PortBase
  {

  public:
    template <class DataType, template <class DataType> class Buffer>
    DataInPort(const char* name, InPort<DataType, Buffer>& inport)
      : PortBase(name)
    {
      // PortProfile::properties 
      addProperty("port.port_type", "DataInPort");

      m_providers.push_back(new InPortCorbaProvider<DataType>(inport));
      m_providers.back()->publishInterfaceProfile(m_profile.properties);
      m_consumers.push_back(new OutPortCorbaConsumer<DataType>(inport));
    }

    virtual ~DataInPort(){};

    /*!
     * @if jp
     * @brief [CORBA interface] Port ³Ԥ
     *
     * InPort  OutPort Ȥ³Ԥ
     *
     * InPort ¦ connect() ǤϰʲΥ󥹤ǽԤ롣
     *
     * 1. InPort ˴Ϣ connector ӥå
     *  - ConnectorProfile::properties["dataport.interface_type"] = CORBA_Any
     *    ξ硢InPortAny ̤ơRawTCP ξ Raw TCP socket ̤
     *    ǡ򴹤롣
     *  - ConnectorProfile::properties["dataport.corba_any.inport_ref"]
     *    InPortAny Υ֥ȥե󥹤򥻥åȤ롣(CORBA_Anyξ)
     *  - ConnectorProfile::properties["dataport.raw_tcp.server_addr"]
     *    InPort¦ΥХɥ쥹򥻥åȤ롣(RawTCP ξ)
     * 2. OutPort˴Ϣ connector μ
     *  - ConnectorProfile::properties["dataport.corba_any.outport_ref"]
     *    OutPortAny Υ֥ȥե󥹤ꤵƤ硢
     *    ե󥹤Consumer֥Ȥ˥åȤ롣
     *    ե󥹤åȤƤʤ̵뤷Ʒ³
     *    (InPortconnect() ƤӽФΥȥݥȤξϡ
     *    OutPortΥ֥ȥե󥹤ϥåȤƤʤϤǤ롣)
     * 3. PortBase::connect() 򥳡
     *    Port³δܽԤ롣
     * 4. 嵭2.OutPortΥե󥹤ǤʤСOutPort
     *    Ϣ connector 롣
     * 5. 嵭νΤĤǤ⥨顼ǤС顼꥿󤹤롣
     *    ˽Ԥ줿RTC::OKǥ꥿󤹤롣
     *  
     * @else
     * @brief [CORBA interface] Connect the Port
     * @endif
     */
 

  protected:
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);

    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);

    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);
    
    struct publish
    {
      publish(SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(InPortProvider* provider)
      {
	provider->publishInterface(m_prop);
      }
      SDOPackage::NVList& m_prop;
    };

    struct subscribe
    {
      subscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortConsumer* consumer)
      {
	consumer->subscribeInterface(m_prop);
      }
      const SDOPackage::NVList& m_prop;
    };

    struct unsubscribe
    {
      unsubscribe(const SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortConsumer* consumer)
      {
	consumer->unsubscribeInterface(m_prop);
      }
      const SDOPackage::NVList& m_prop;
    };

  private:
    struct find_conn_id;
    std::vector<InPortProvider*> m_providers;
    std::vector<OutPortConsumer*> m_consumers;
  };
}; // namespace RTC

#endif // DataInPort_h
